import org.junit.Assert;
import org.junit.Test;

import java.io.FileFilter;
import java.io.File;
import java.util.Collection;
import java.util.List;

public class PublicTest {

	public static String CITY_GRAPH_FILE_PATH;

    public static String CITY_GRAPH_RAND_FILE_PATH;

    static {
        File f = new File("testdata");
        if (!f.exists()) {
            File[] files = (new File(System.getProperty("user.dir"))).listFiles(new FileFilter() {
                public boolean accept(File file) {
                    return file.isDirectory() && !file.getName().contains("bin");
                }
            });
            if (files != null && files.length > 0)
                f = new File(files[0], "testdata");
        }
        File fCities = new File(f, "Cities.csv");
        File rCities = new File(f, "randData.csv");
        CITY_GRAPH_FILE_PATH = fCities.getAbsolutePath();
        CITY_GRAPH_RAND_FILE_PATH = rCities.getAbsolutePath();
    }

    private static final double ASSERT_DELTA = 0.01;

    @Test
    public void check_city_class() {
        // Name
        ICity city1 = CityFactory.createCity("Test1");
        ICity city11 = CityFactory.createCity("Test1");
        ICity city2 = CityFactory.createCity("Test2");
        ICity city3 = CityFactory.createCity("Test3");

        Assert.assertEquals("getName() gibt nicht den richtigen Namen zurueck", "Test1", city1.getName());
        Assert.assertEquals("toString() gibt nicht den Namen der Stadt zurueck", "Test1", city1.toString());

        // Equals
        Assert.assertEquals("equals() ist nicht true falls der Name der Staedte gleich ist", city11, city1);

        // Distance
        city1.setDistance(10.11);
        Assert.assertEquals("getDistance() gibt nicht die richtige Distanz zurueck", 10.11, city1.getDistance(), ASSERT_DELTA);

        //Predecessor
        city1.setPredecessor(city2);
        Assert.assertEquals("getPredecessor() gibt nicht die richtigen Vorgaenger zurueck", city2.getName(), city1.getPredecessor().getName());

        //reset
        city1.reset();
        Assert.assertEquals(Double.MAX_VALUE, city1.getDistance(), ASSERT_DELTA);
        Assert.assertEquals(null, city1.getPredecessor());

        // connections
        Connection c1 = new Connection(city2, 2);
        Connection c2 = new Connection(city3, 3);

        city1.addConnection(c1);
        city1.addConnection(c2);

        for (Connection in : city1.getConnections()) {
            Assert.assertTrue(in.equals(c1) || in.equals(c2));
        }
        Assert.assertEquals(2, city1.getConnections().size());

    }

    private static String res1 = "Test0, Test1, Test2, Test3, Test4, Test5, Test6, Test7, Test8, Test9; distance: 45.0 km.";

    @Test
    public void check_route_class() {
        ICity[] cities = new City[10];
        double dist = 0;
        for (int i = 0; i < 10; i++) {
            cities[i] = CityFactory.createCity(String.format("Test%d", i));
        }

        IRoute r = CityFactory.createRoute(cities[0]);
        for (int i = 1; i < 10; i++) {
            r.addConnectionToRoute(new Connection(cities[i], (double) i));
            dist += i;
        }
        Assert.assertEquals("getLength() gibt nicht die richtige  Laenge zurueck", r.getLength(), dist, ASSERT_DELTA);
        Assert.assertEquals("toString() entspricht nicht dem spezifizierten Pattern", r.toString(), res1);

        Assert.assertEquals("getOrigin() gibt nicht den richtigen Startpunkt zurueck", r.getOrigin().getName(), cities[0].getName());

        int count = 1;
        for (Connection cur : r.getConnections()) {
            Assert.assertEquals(cur.getDestination().getName(), cities[count].getName());
            Assert.assertEquals(cur.getDistance(), ((double) count), ASSERT_DELTA);
            count++;
        }

    }

    private static Graph graph = GraphBuilder.buildGraphFromFile(CITY_GRAPH_FILE_PATH);
    private static Graph graphRand = GraphBuilder.buildGraphFromFile(CITY_GRAPH_RAND_FILE_PATH);

    private static String citiesData = "Basel;Bellinzona;Bern;Chur;Genf;Interlaken;Kreuzlingen;Lausanne;Locarno;Lugano;Luzern;Neuenburg;Sankt Gallen;Schaffhausen;Thun;Winterthur;Zug;Zurich";
    private static String[] cities = citiesData.split(";");

    private static String solBasel = "0.00,239.90,101.00,207.20,257.30,160.30,154.80,194.00,263.00,270.50,99.90,128.00,174.60,98.80,129.50,112.50,120.60,88.20";
    private static String solZurich = "88.20,203.50,125.00,119.00,281.30,131.70,66.60,218.00,226.60,234.10,63.50,176.60,86.40,52.70,153.50,24.30,32.40,0.00";
    private static String solLausanne = "194.00,331.60,93.00,333.70,63.30,152.30,284.60,0.00,354.70,362.20,191.60,72.80,304.40,270.70,121.50,242.30,222.70,218.00";
    private static String solLocarno = "263.00,23.10,261.70,139.10,418.00,231.30,267.10,354.70,0.00,53.70,163.10,313.30,242.10,279.30,262.10,250.90,194.20,226.60";

    private static String solRandc12 = "28.00,22.00,27.00,47.00,102.00,10.00,48.00,46.00,46.00,102.00,57.00,50.00,0.00,43.00,41.00,45.00,39.00,72.00,54.00,108.00," +
            "71.00,56.00,38.00,30.00,73.00,25.00,94.00,26.00,59.00,52.00,89.00,87.00,61.00,130.00,95.00,41.00,47.00,46.00,63.00,101.00,96.00,61.00,41.00,105.00," +
            "94.00,58.00,66.00,64.00,63.00,55.00";

    private static String solRandc37 = "49.00,48.00,53.00,58.00,91.00,36.00,37.00,65.00,77.00,74.00,11.00,69.00,46.00,69.00,52.00,58.00,28.00,54.00,81.00,62.00," +
            "35.00,10.00,27.00,63.00,37.00,46.00,89.00,52.00,50.00,13.00,94.00,71.00,56.00,85.00,87.00,67.00,1.00,0.00,17.00,95.00,122.00,33.00,30.00,76.00,58.00," +
            "65.00,69.00,45.00,77.00,81.00";

    private static String solRandc37Limited60 = "49.00,48.00,53.00,58.00,158.00,36.00,37.00,65.00,77.00,74.00,11.00,69.00,46.00,69.00,52.00,58.00,28.00,54.00,81.00," +
            "144.00,35.00,10.00,27.00,63.00,37.00,46.00,107.00,52.00,50.00,13.00,102.00,71.00,56.00,215.00,87.00,67.00,1.00,0.00,17.00,99.00,122.00,33.00,30.00,99.00," +
            "58.00,65.00,69.00,45.00,77.00,81.00";

    private static String solRandc16Limited60 = "52.00,54.00,25.00,66.00,130.00,42.00,9.00,37.00,76.00,94.00,39.00,41.00,39.00,56.00,60.00,30.00,0.00,82.00,53.00,164.00," +
            "34.00,38.00,1.00,35.00,36.00,49.00,79.00,26.00,57.00,15.00,74.00,99.00,28.00,187.00,70.00,41.00,29.00,28.00,45.00,71.00,94.00,53.00,2.00,119.00,86.00,64.00,41.00," +
            "62.00,49.00,53.00";

    private static double[] transform(String sol) {
        String[] items = sol.split(",");
        double[] res = new double[items.length];
        for (int iter = 0; iter < res.length; iter++) {
            res[iter] = Double.valueOf(items[iter]);
        }
        return res;
    }

    private Connection checkConnectionExists(Collection<Connection> conn, String name) {
        for (Connection c : conn)
            if (c.getDestination().getName().equals(name))
                return c;
        return null;
    }

    private void checkRouteValid(Graph g, IRoute route, String from, String to) {
        List<Connection> connections = route.getConnections();
        ICity cur = g.getCity(route.getOrigin().getName());
        Assert.assertEquals("Startpunkt der Route stimmt nicht", cur.getName(), from);
        for (Connection conn : connections) {
            Connection real = checkConnectionExists(cur.getConnections(), conn.getDestination().getName());
            Assert.assertTrue(real != null);
            Assert.assertEquals(conn.getDistance(), real.getDistance(), ASSERT_DELTA);
            cur = g.getCity(conn.getDestination().getName());
        }
        Assert.assertEquals("Ziel der Route stimmt nicht", cur.getName(), to);
    }

    private double roundTest(double in) {
        return ((double) ((long) (in * 100))) / 100;
    }

    private void checkSol(String start, String solString) {
        double[] solution = transform(solString);
        for (int i = 0; i < cities.length; i++) {
            IRoute r = graph.calculateShortestPath(start, cities[i]);
            checkRouteValid(graph, r, start, cities[i]);
            Assert.assertTrue("Nicht die kuerzeste Route!", roundTest(r.getLength()) <= solution[i]);
        }
    }

    private void checkSol(Graph graph, String start, String solString, int max) {
        double[] solution = transform(solString);
        for (int i = 0; i < max; i++) {
            IRoute r = graph.calculateShortestPath(start, "c" + i);
            checkRouteValid(graph, r, start, "c" + i);
            Assert.assertTrue("Nicht die kuerzeste Route!", roundTest(r.getLength()) <= solution[i]);
        }
    }

    private void checkSolLimited(Graph graph, String start, String solString, int max, int limit) {
        double[] solution = transform(solString);
        for (int i = 0; i < max; i++) {
            IRoute r = graph.calculateShortestPathLimited(start, "c" + i, limit);
            checkRouteValid(graph, r, start, "c" + i);
            Assert.assertTrue("Nicht die kuerzeste Route!", roundTest(r.getLength()) <= solution[i]);
        }
    }

    @Test
    public void check_dist1() {
        checkSol("Basel", solBasel);
    }

    @Test
    public void check_dist2() {
        checkSol("Lausanne", solLausanne);
    }

    @Test
    public void check_dist3() {
        checkSol("Zurich", solZurich);
    }

    @Test
    public void check_dist4() {
        checkSol("Locarno", solLocarno);
    }

    @Test
    public void check_dist_rand_1() {
        checkSol(graphRand, "c37", solRandc37, 50);
    }

    @Test
    public void check_dist_rand_2() {
        checkSol(graphRand, "c12", solRandc12, 50);
    }

    private static void addConnections(ICity c1, ICity c2, double dist) {
        c1.addConnection(new Connection(c2, dist));
        c2.addConnection(new Connection(c1, dist));
    }

    private static Graph createLimitedGraph(int len, int dist) {
        assert (len >= 3);
        ICity[] cities = new ICity[len];
        Graph g = new Graph();
        for (int i = 0; i < cities.length; i++) {
            cities[i] = CityFactory.createCity("C_" + i);
            g.putCity("C_" + i, cities[i]);
        }

        addConnections(cities[0], cities[cities.length - 1], ((cities.length - 1) * dist) - 1);
        for (int i = 0; i < cities.length - 1; i++) {
            addConnections(cities[i], cities[i + 1], dist);
        }
        return g;
    }

    @Test
    public void check_dist_limited() {
        Graph g = createLimitedGraph(10, 2);
        IRoute r = g.calculateShortestPathLimited("C_0", "C_9", 3);
        checkRouteValid(g, r, "C_0", "C_9");
        Assert.assertEquals("Not a limited shortest path ", 9, r.getConnections().size());
        Assert.assertEquals("Not a limited shortest path ", 18.0, r.getLength(), ASSERT_DELTA);

    }

    @Test
    public void empty_route() {
        ICity c1 = CityFactory.createCity("Test1");
        Graph g = new Graph();
        g.putCity("Test1", c1);
        IRoute r = g.calculateShortestPath("Test1", "Test1");
        Assert.assertEquals("No route.", r.toString());
    }

    @Test(expected = NoRouteFoundException.class)
    public void basic_nopath() {
        ICity c1 = CityFactory.createCity("Test1");
        ICity c2 = CityFactory.createCity("Test2");
        Graph g = new Graph();
        g.putCity("Test1", c1);
        g.putCity("Test2", c2);
        IRoute r = g.calculateShortestPath("Test1", "Test2");
    }

    @Test(expected = IllegalArgumentException.class)
    public void check_wrong_city() {
        graph.calculateShortestPath("BLI", "c1");
    }

    @Test
    public void check_dist_limited_rand_1() {
        checkSolLimited(graphRand, "c16", solRandc16Limited60, 50, 60);
    }

    @Test
    public void check_dist_limited_rand_2() {
        checkSolLimited(graphRand, "c37", solRandc37Limited60, 50, 60);
    }
}
